define(['underscore', 'backbone', 'context'], function (_, Backbone, Context) {
    return Backbone.Model.extend({
        url_base: "/schema/%(connection)s/%(db)s/%(table)s",
        url_base_create: "/schema/column/create/%(connection)s/%(db)s/%(table)s",
        url: function () {
            return encodeURI(this.url_base.format(Context));
        },
        setCreateURL: function () {
            this.url = encodeURI(this.url_base_create.format(Context));
        },
        setContext: function (localContext) {
            this.url = encodeURI(this.url_base.format(localContext));
        },
        parse: function (schema) {
          // assure that meta data is in title case
          // redshift always returns columns in lower case
          if (Context.isRedshift() && !_.isEmpty(schema.meta)) {
            schema.meta = _.map(schema.meta, function (meta) {
              return _.object(_.map(meta, function (item, column) {
                return [Context.titleCase(column), item];
              }));
            });
          }

          if (Context.isRedshift() && !_.isEmpty(schema.indexes)) {
            schema.indexes = _.map(schema.indexes, function (index) {
              return _.object(_.map(index, function (item, key) {
                return [Context.titleCase(key), item];
              }));
            });
          }

          return schema;
        },
        defaults: {"meta": [], "charsets": []}
//        actual schema for each column
//            Default: null,
//            Collation: null,
//            Extra: null,
//            Field: null,
//            Privileges: null,
//            Null: null,
//            Comment: null,
//            Type: null,
//            Key: null
    });
});
