define(['jquery', 'underscore', 'backbone', 'views/connections', 'views/sidebar', 'models/connection',
        'views/content/query', 'context', 'views/content/output', 'tpls', 'views/newconnection', 'events',
         'anywhere', 'baseview'],
    function ($, _, Backbone, Connections, SidebarView, Connection, QueryView, Context, OutputView, Templates,
              NewConnection, Events, Anywhere, BaseView) {
        var ContentView = BaseView.extend({
            template: Templates["content/body"],
            _views: {},
            anywhere: null,
            initialize: function () {

                // bind methods to this context
                _.bindAll(this,
                    "renderSubViews",
                    "connectionError",
                    "renderNewConnection",
                    "renderConnectionBar",
                    "connectionRemoved",
                    "renderMainContent",
                    "renderEditConnection",
                    "closeEditForm",
                    "showOutputPanel",
                    "updateOutputPanel");

                // add event listeners
                this.addListeners();

                // set initial connection model
                this.connection = new Connection({id: Context.connection});
            },
            events: {
                "click .close-btn": "closeEditForm"
            },
            addListeners: function () {
                this.listenTo(Events, 'connection:edit', this.renderEditConnection);
                this.listenTo(Events, 'connection:create', this.renderConnectionBar);
                this.listenTo(Events, 'connection:change', this.renderSubViews);
                this.listenTo(Events, 'connection:new', this.renderNewConnection);
                this.listenTo(Events, 'connection:remove', this.connectionRemoved);
                this.listenTo(Events, "connections:fetched", this.renderMain);
                this.listenTo(Events, 'query:fetching', this.showOutputPanel);
                this.listenTo(Events, 'tables:fetched', this.updateAnywhere);
                this.listenTo(Events, 'db:change', this.updateAnywhere);
                this.listenTo(Events, 'queryMode:change', this.updateOutputPanel);
                this.listenTo(Events, 'db:change', this.updateOutputPanel);
                this.listenTo(Events, 'route:user:account', this.updateOutputPanel);
                this.listenTo(Events, 'connection:error', this.connectionError);
            },
            updateOutputPanel: function () {

                if (Context.query_mode === "structure" || Context.query_mode === "backups") {
                    return this.hideOutputPanel();
                }

                // on db:change event if table is null, we hide output panel
                if (Context.table === null) {
                    return this.hideOutputPanel();
                }
            },
            anywhereCallbacks: {
                databases: function (name) {
                    Events.trigger("db:navigate", name);
                },
                tables: function (name) {
                    Events.trigger("table:navigate", name);
                },
                create: function (name) {
                    Events.trigger("create:navigate", name);
                },
                connections: function (conn) {
                    Events.trigger("connection:navigate", conn.id);
                }
            },
            initAnywhere: function () {
                // initialize sensei anywhere
                var limitPerGroup = 5;
                this.anywhere = $.anywhere([], ["command+shift+k", "ctrl+shift+k"], limitPerGroup, true);
                var callbacks = this.anywhereCallbacks;
                this.anywhere.events.on("select", function (item, data) {
                    var type = data.group;
                    if (_.has(callbacks, type)) {
                        callbacks[type](item);
                    }
                });
            },
            updateAnywhere: function () {

                // lazy initialize anywhere plugin
                if (this.anywhere === null) {
                    this.initAnywhere();
                }

                // default value for connections
                var conns = null;

                // check if connections are available
                if (Context.connections && Context.connections.connections) {
                  conns = Context.connections.connections;
                }

                // create data array with tables and dbs
                var data = [
                    {
                        group: "tables",
                        items: Context.tables
                    },
                    {
                        group: 'databases',
                        items: Context.dbs
                    },
                    {
                        group: 'create',
                        items: [
                            {title: "Create database", target: "createDatabaseModal"}, 
                            {title: "Create table", target: "createTableModal"}
                        ],
                        customFormatter: function (item) {
                            return item.title;
                        }
                    },
                    {
                        group: 'connections',
                        items: conns,
                        customFormatter: function (conn) {
                            var tpl = Handlebars.compile('{{title}} <span class="sa-custom-item-meta">{{username}}@{{hostname}}:{{port}}</span>');
                            return tpl(conn);
                        }
                    }
                ];

                // set updated data in anywhere
                this.anywhere.setData(data);
            },
            connectionRemoved: function (model) {
                if (model.id === this.connection.id) {
                    this.renderNewConnection();
                }
            },
            connectionError: function (model, xhr) {
                var error = "Connection doesn't exist!";
                if (xhr.responseJSON && xhr.responseJSON.error) {
                    error = xhr.responseJSON.error;

                    // set current connection model anyway
                    this.connection = model;
                    // render edit connection form so the user has a chance to edit faulty connection
                    this.renderEditConnection(true);
                } else {
                    // throw regular error on 404
                    alert(error);
                }

                console.warn(error, arguments, this.connection);
            },
            renderNewConnection: function () {
                console.info("renderNewConnection");
                this.$(".create-new-connection-tab").addClass("active");
                //this.clearSubViews();
                //this.hideSubviews();
                this.$(".main-wrapper").hide();
                this._views.new = new NewConnection();
                this.$(".overlay-wrapper").html(this._views.new.render().el);

                // update URL
                Backbone.history.navigate("/content/filter/table/0");
            },
            renderEditConnection: function (showConnectionError) {
                console.info("renderEditConnection", this.connection);

                // check if connection has id but no hostname, it means the connection is faulty
                // in this case we need to get the data from connection list
                var id = this.connection.get("id");
                if (id && !this.connection.get("hostname")) {
                    // alias the connections list
                    var conns = Context.connections.connections;
                    // try to find connection data in connection list
                    var conn = _.findWhere(conns, {id: id});
                    // check if we found anything
                    if (conn) {
                        // set data from connection list to the current connection instance
                        this.connection.set(conn);
                    }
                }

                this._views.edit = new NewConnection({model: this.connection,
                    showConnectionError: showConnectionError, modal: !showConnectionError});

                if (showConnectionError) {
                    this.$(".overlay-wrapper").show();
                    this.$(".main-wrapper").hide();
                    var view = this._views.edit.render().el;
                    this.$(".overlay-wrapper").html(view);
                } else {
                    //this.$(".overlay-wrapper").show();
                    //this.$(".main-wrapper").hide();
                    this.$(".edit-connection-modal").modal();
                    var view = this._views.edit.render().el;
                    //this.$(".overlay-wrapper").html(view);
                    this.$(".edit-connection-modal .modal-content").html(view);
                }
            },
            renderAccount: function () {

            },
            closeEditForm: function (e) {
                e.preventDefault();
                this.hideEditConnection();
            },
            hideEditConnection: function (e) {
                if (e) {
                    // do not hide the form is the target is the form itself
                    var $parentForm = $(e.target).parents(".new-connection");
                    if ($parentForm.length > 0 || $(e.target).is(".new-connection")) {
                        return;
                    }
                }

                this.$(".edit-connection-modal").modal("hide");
                this.$(".overlay-wrapper").html("");
                this.$(".main-wrapper").show();
            },
            clearSubViews: function () {

                // hide edit connection if not closed in other tab
                this.hideEditConnection();

                if (!_.isEmpty(this._views)) {

                    // wow, all events are unbinded here
                    // @FIXME URGENT!
                    //Events.unbind();
                    //console.warn("here would be da unbind");

                    _.each(this._views, function (subview) {
                        //subview.stopListening();
                        subview.remove();
                    });
                    this._views = {};

                    //this.addListeners();
                }
            },
            resetContext: function (model) {
                Context.connection = model ? model.id : null;
                Context.driver = model ? model.get("driver") : null;
                Context.table = null;
                Context.db = null;
            },
            renderSubViews: function (model) {

                // bind model
                this.connection = model;

                // reset state if context has already been loaded
                if (Context.loaded) {
                    this.resetContext(model);
                }

                // set default db if one is defined
                // @todo move to resetContext
                if (Context.db === null && model.get("db") !== null) {
                    Context.db = model.get("db");
                }

                // set driver
                // @todo -//-
                Context.driver = model.get("driver");

                console.info("renderSubViews", Context, model.toJSON());

                // sync url
                Backbone.history.syncURL();

                // clear existing sub views and rebind events
                this.clearSubViews();

                // load sidebar
                this._views.sidebar = new SidebarView();
                var data = {connection: model.toJSON()};
                this.$(".sidebar").html(this._views.sidebar.render(data).el);

                // render main content views
                this.renderMainContent();
            },
            mainContent: function () {
                return this.$(".query-panel, .output-panel");
            },
            showOutputPanel: function () {
                this.$(".output-panel").show();
            },
            hideOutputPanel: function () {
                this.$(".output-panel").hide();
            },
            renderMainContent: function () {

                // load query panel
                this._views.query = new QueryView();
                this.$(".query-panel").html(this._views.query.render().el);

                if (Context.output) {
                    // load content panel
                    this._views.output = new OutputView();
                    this.$(".output-panel").html(this._views.output.render().el);
                }
            },
            renderConnectionBar: function (model) {

                // reset connection bar
                if (this.connections) {
                    this.connections.remove();
                }

                // if model is passed, set active connection
                if (model) {
                    this.connection = model;
                }

                // render connection bar and set active connection
                this.connections = new Connections({active: this.connection.id});
                this.$(".connections-bar").html(this.connections.render().el);
            },
            renderMain: function () {
                // @fixme re-enable back!

                // @todo quickfix, this gets set in constructor too, need to refactor
                this.connection = new Connection({id: Context.connection});

                //console.warn("context now", this.connection, Context.connection);
                //this.connection.fetch({success: this.renderSubViews, error: this.connectionError});

                if (_.isNumber(Context.connection)) {
                    if (Context.connection !== 0) {
                        // fetch active connection
                        this.connection.fetch({success: this.renderSubViews, error: this.connectionError});
                    } else {
                        this.renderNewConnection();
                    }
                }
            },
            render: function () {

                console.info("[!] render content view");

                // load body template
                this.$el.html(this.template());

                // render main view
                //this.renderMain();

                // load connections bar
                this.renderConnectionBar();

                return this;
            }
        });
        return ContentView;
    });
