define(['jquery', 'underscore', 'backbone', 'tpls', 'events',
      'collections/history/queries', 'models/history/query',
      'views/content/query/history/item', 'baseview', 'context',
      'models/history/query', 'collections/history/favorites',
      'models/backup'],
    function ($, _, Backbone, Templates, Events, HistoryQueries, HistoryQuery,
              HistoryItem, BaseView, Context, QueryModel, FavoriteQueries, Backup) {
      var HistoryList = BaseView.extend({
        template: Templates["content/query/history/list"],
        queries: null,
        parent: null,
        initialize: function (opts) {

          this.parent = opts.parent;
          this.queries = new HistoryQueries();
          this.favorites = new FavoriteQueries();
          this.listenTo(this.queries, "add", this.addQuery, this);
          this.listenTo(this.favorites, "add", this.addFavorite, this);
          this.listenTo(Events, "db:change", this.resetLists, this);
          // this.listenTo(this.queries, "request", this.resetHistoryList, this);

          _.bindAll(this, "queriesFetched", "favoritesFetched", "addQuery",
              "addFavorite", "resetLists", "addEmptyMessage");
        },
        events: {
          // export actions
          "click .export-query": "exportQueryModal",
          "submit .export-modal form": "exportQuery",
          // favorite actions
          "click .save-favorite": "favoriteModal",
          "submit .save-favorite-modal form": "saveFavorite"
        },
        exportQueryModal: function (e) {
          if (e) {
            e.preventDefault();
          }

          var query = this.getExportQuery();

          this.$(".export-modal").modal();
          this.$(".export-modal .query-wrapper").text(query);
        },
        getExportQuery: function () {
          var query;
          if (!_.isEmpty(Context.query)) {
            query = Context.query;
          } else if (Context.query_mode === "raw" && !_.isEmpty(Context.lastRawQuery)) {
            query = Context.lastRawQuery;
          }
          return query;
        },
        exportQuery: function (e) {
          if (e) {
            e.preventDefault();
          }

          // create new instance of backup model
          var backup = new Backup();
          var query = this.getExportQuery();


          if (_.isEmpty(query)) {
            alert("Can't export data, query is not available.");
            return;
          }

          var $btn = this.$(".export-modal .btn-primary:first");
          var prevText = $btn.text();
          $btn.prop("disabled", true).text("Exporting...");

          // set data object, get last query
          var data = {conn: Context.connection, db: Context.db, query: query};

          // change model url to export endpoint
          backup.url = "/backup/query";

          // call backup save method, which will start query export
          backup.save(data, {
            success: function (model, res) {
              $btn.prop("disabled", false).text(prevText);
              console.log("backup.save->success", arguments);
              // check if response has download url
              if (res && !_.isEmpty(res.url)) {
                // response correct, redirect to download url
                window.location.replace(res.url);
              } else {
                // response invalid, show error
                alert("Failed to create CSV file.")
              }
            },
            error: function () {
              $btn.prop("disabled", false).text(prevText);
              console.log("backup.save->error", arguments);
              // query export request failed, show error
              alert("Failed to export query.")
            }
          });
        },
        remove: function () {
          // clean dropdowns and tooltips
          $("li>a", this.$queryHistory).tooltip('destroy');
          $("li>a", this.$favoriteQueries).tooltip('destroy');
          this.$queryHistory.remove();
          this.$favoriteQueries.remove();

          BaseView.prototype.remove.call(this);
        },
        resetLists: function () {
          // need to delete all subviews on db change, entries will be
          // fetched on dropdown activation
          _.each(this.subviews, function (view) {
            view.remove();
          });

          this.addEmptyMessage();
        },
        addEmptyMessage: function () {

          // remove existing empty messages
          $(".empty", this.$queryHistory).remove();
          $(".empty", this.$favoriteQueries).remove();

          // add empty placeholders before fetch occurs
          if ($(".empty:visible", this.$queryHistory).length === 0 &&
              this.queries.length === 0) {

            this.$queryHistory.append('<li class="empty"><a>' +
                'No recent queries found.</a></li>');

          }
          if ($(".empty:visible", this.$favoriteQueries).length === 0 &&
              this.favorites.length === 0) {

            this.$favoriteQueries.append('<li class="empty"><a>' +
                'No favorites found.</a></li>');

          }
        },
        favoriteModal: function (e) {
          if (e) {
            e.preventDefault();
          }
          this.$(".save-favorite-modal").modal();
          this.$(".save-favorite-modal input:first").focus();
        },
        saveFavorite: function (e) {
          if (e) {
            e.preventDefault();
          }

          var $modal = this.$(".save-favorite-modal");
          var $nameInput = $("input[name=name]", $modal);
          var $team_shared = $("input[name=team_shared]", $modal);

          var data = {};
          data.name = $nameInput.val();
          data.conn_id = Context.connection;
          data.db = Context.db;
          data.table = Context.table;
          data.is_favorite = true;
          data.team_shared = $team_shared.prop("checked");

          // check if parent view exists
          if (this.parent && this.parent.getQuery) {
            // get query from parent view
            data.query = this.parent.getQuery();
          } else {
            // fallback to current query if no parent can be reached
            data.query = Context.query;
          }

          // hash should be the same as query
          data.hash = data.query;

          var query = new QueryModel();
          query.save(data, {
            success: function () {
              $modal.modal("hide");
              $nameInput.val(null);
            }, error: function () {
              alert("Failed to save query to favorites.");
            }
          });

          console.log("HistoryList.saveFavorite", data);
        },
        render: function () {
          this.$el.html(this.template());

          // reference dropdown menu, because it is detached in later stages
          this.$queryHistory = this.$(".query-history");
          this.$favoriteQueries = this.$(".favorite-queries");

          // setup dropdown menu for query history
          this.$(".dropdown-toggle").dropdown();

          // must use .history-btn because events are fired on dropdown
          // parent element
          var that = this;
          this.$(".history-btn").on('show.bs.dropdown', function () {
            if (that.queries !== null) {
              that.queries.fetch({success: that.queriesFetched});
            }
          });

          this.$(".favorites-btn").on('show.bs.dropdown', function () {
            if (that.favorites !== null) {
              that.favorites.fetch({success: that.favoritesFetched});
            }
          });

          return this;
        },
        queriesFetched: function () {
          $("li>a", this.$queryHistory).tooltip({
            container: 'body'
          });
          this.addEmptyMessage();
        },
        favoritesFetched: function () {
          $("li>a", this.$favoriteQueries).tooltip({
            container: 'body'
          });
          this.addEmptyMessage();
        },
        addQuery: function (query) {
          if ($(".empty", this.$queryHistory).length > 0) {
            $(".empty", this.$queryHistory).remove();
          }
          this.subviews[query.cid] = new HistoryItem({model: query});
          $(".dropdown-header", this.$queryHistory).after(this.subviews[query.cid].render().el);
        },
        addFavorite: function (query) {
          if ($(".empty", this.$favoriteQueries).length > 0) {
            $(".empty", this.$favoriteQueries).remove();
          }
          this.subviews["favorite-" + query.cid] = new HistoryItem({model: query});
          $(".dropdown-header", this.$favoriteQueries).after(this.subviews["favorite-" + query.cid].render().el);
        }
      });
      return HistoryList;
    });
