define(['jquery', 'underscore', 'backbone', 'tpls', 'views/sidebar/tables', 'views/sidebar/databases', 'events',
    'baseview', 'models/table', 'context', 'models/database'],
    function ($, _, Backbone, Templates, TablesView, DatabasesView, Events, BaseView, Table, Context, Database) {
        return BaseView.extend({
            template: Templates["sidebar"],
            table: null, // table model
            dropDatabaseConfirm: "Are you sure you want to drop this database?",
            events: {
                "click .edit-connection": "editConnection",
                "click .create-table": "createTableModal",
                "click .create-database": "createDatabaseModal",
                "click .drop-database": "dropDatabase",
                "submit .create-table-modal form": "createTable",
                "submit .create-database-modal form": "createDatabase"
            },
            initialize: function () {
                _.bindAll(this, "createDatabaseModal", "createTableModal", "dropDatabase");
                this.listenTo(Events, "create:navigate", this.navigateCreate);
                this.listenTo(Events, "table:rename", this.createTableModal);
            },
            navigateCreate: function (obj) {
                if (this[obj.target]) {
                    this[obj.target]();
                }
            },
            editConnection: function (e) {
                e.preventDefault();
                Events.trigger("connection:edit");
            },
            // createTableModal: function (e) {
            //     if (e) { e.preventDefault(); }
            //     this.$(".create-table-modal").modal();
            //     this.$(".create-table-modal input:first").focus();
            // },
            createTableModal: function (e, model) {
                if (e) { e.preventDefault(); }
                this.$(".create-table-modal").modal();
                this.$(".create-table-modal input[name=title]").focus();

                if (model) {
                    this.table = model;
                    this.$(".create-table-modal input[name=title]").val(model.get("title"));
                    this.$(".create-table-modal input[name=rename_from]").val(model.get("title"));
                    this.$(".create-table-modal h4").text("Rename table");
                } else {
                    this.$(".create-table-modal input[name=title]").val("");
                    this.$(".create-table-modal input[name=rename_from]").val("");
                    this.$(".create-table-modal h4").text("Create table");
                }
            },
            createDatabaseModal: function (e) {
                if (e) { e.preventDefault(); }
                this.$(".create-database-modal").modal();
                this.$(".create-database-modal input:first").focus();
            },
            createTable: function (e) {
                if (e) { e.preventDefault(); }

                var $modal = this.$(".create-table-modal");
                $(".error", $modal).remove();

                var data = Context.serializeObj($(e.target));

                // force create new table model if we are not renaming
                if (_.isEmpty(data["rename_from"])) {
                    var table = new Table();        
                } else {
                    var table = this.table || new Table();                 
                }

                table.save(data, {success: function (model) {

                    var prevContextTable = Context.table;

                    // update context on new table creation
                    // or if renaming current table
                    if (_.isEmpty(model.get("rename_from")) 
                        || prevContextTable === model.get("rename_from")) {
                        Context.table = model.get("title");
                    }

                    // this will trigger adding dom element to sidebar
                    Events.trigger("table:create", model);

                    if (_.isEmpty(model.get("rename_from"))) {
                        // trigger table:created event only on new table
                        Events.trigger("table:created", model);
                    } else {
                        // trigger table:renamed event
                        Events.trigger("table:renamed", model);
                    }

                    // general after-event
                    Events.trigger("table:changed", model);

                    // navigate to table if a new table is created or renamed table was previously active
                    if (_.isEmpty(model.get("rename_from")) 
                        || prevContextTable === model.get("rename_from")) {
                        Events.trigger("table:navigate", model.get("title"));
                    }

                    // reset modal
                    $("input", $modal).val("");
                    $modal.modal("hide");

                }, error: function (model, xhr) {

                    var res = null;

                    try {
                        res = $.parseJSON(xhr.responseText);
                    } catch (e) {
                        alert("Unexpected error.");
                    }

                    if (res && _.isString(res.error)) {
                        var $error = $("<p>").addClass("error help-block").text(res.error);
                        $error.insertAfter($("input[name=title]", $modal));
                    }
                }});

            },
            createDatabase: function (e) {
                if (e) { e.preventDefault(); }

                var $modal = this.$(".create-database-modal");
                $(".error", $modal).remove();

                var data = Context.serializeObj($(e.target));
                var db = new Database();
                var that = this;

                db.save(data, {success: function (model) {
                    console.log("db.save", model.toJSON(), model.get("title"));
                    Context.db = model.get("title");
                    that.renderDatabases();
                    Events.trigger("db:create", model);
                    // Events.trigger("db:navigate", model.get("title"));
                    Events.trigger("db:created", model);
                    $("input[name=title]", $modal).val("");
                    $("select[name=charset] option", $modal).prop("selected", false);
                    $modal.modal("hide");
                }, error: function (model, xhr) {

                    var res = null;

                    try {
                        res = $.parseJSON(xhr.responseText);
                    } catch (e) {
                        alert("Unexpected error.");
                    }

                    if (res && _.isString(res.error)) {
                        var $error = $("<p>").addClass("error help-block").text(res.error);
                        $error.insertAfter($("input[name=title]", $modal));
                    }
                }});

            },
            dropDatabase: function (e) {
                if (e) { e.preventDefault(); }

                if (confirm(this.dropDatabaseConfirm)) {
                    var db = new Database({title: Context.db});
                    var that = this;
                    db.destroy({success: function () {

                        // reset context
                        Context.db = null;

                        Events.trigger("db:dropped", db);

                        // update url
                        // Backbone.history.syncURL();

                        // reload database list
                        // that.renderDatabases();

                    }, error: function () {
                        alert("Unexpected error: failed to drop database.");
                    }});
                }
            },
            render: function (data) {

                // add charsets to view's data
                data.charsets = Context.charsets;

                this.$el.html(this.template(data));

                this.renderDatabases();
                this.renderTables();

                // manually context menu bind events, because context menu gets detached
                this.$(".db-row .create-database").click(this.createDatabaseModal);
                this.$(".db-row .create-table").click(this.createTableModal);
                this.$(".db-row .drop-database").click(this.dropDatabase);

                return this;
            },
            renderDatabases: function () {
                this.subviews["dbs"] = new DatabasesView();
                this.$(".db-switcher").html(this.subviews["dbs"].render().el);
            },
            renderTables: function () {
                this.subviews["tables"] = new TablesView();
                this.subviews["tables"].setElement(this.$(".tables-list"));
            }
        });
});

