define(["jquery", "underscore", "backbone", "tpls", "events", "models/user", "context"],
    function ($, _, Backbone, Templates, Events, User, Context) {
        return Backbone.View.extend({
            template: Templates["user/account"],
            subviews: {},
            initialize: function () {
                // initialize user model
                this.model = new User();
            },
            events: {
                "submit form": "saveForm"
            },
            saveForm: function (e) {
                e.preventDefault();

                var data = Context.serializeObj($(e.currentTarget));

                this.$("form .alert-success").fadeOut();
                this.$("p.error").remove();

                this.model.save(data, {
                    success: function (model) {
                        this.$("form .alert-success").fadeIn();
                        Events.trigger('account:save', model);
                    },
                    error: function (model, xhr) {

                        var res = $.parseJSON(xhr.responseText);

                        if (_.isObject(res.errors)) {
                            _.each(res.errors, function (error, key) {
                                var $error = $("<p>").addClass("error help-block").text(error);
                                var $field = this.$("input").filter(function () { 
                                    return $(this).attr("name") === key; 
                                });
                                $error.insertAfter($field);
                            }, this); 
                        }
                    }
                });
            },
            render: function () {

                // render template
                this.$el.html(this.template());

                // loading indicator
                this.$(".loading-icon-wrapper").show();
                this.$("fieldset").hide();

                this.model.fetch({
                    success: this.setData,
                    error: function () {
                        alert("Failed to fetch user account");
                    }
                });
                return this;
            },
            setData: function (model, data) {

                // loading indicator
                this.$(".loading-icon-wrapper").hide();
                this.$("fieldset").show();

                // fill form data
                this.$("input[name=name]").val(model.get("name"));
                this.$("input[name=email]").val(model.get("email"));
            }
        });
    });
