define(['jquery', 'underscore', 'backbone', 'handlebars', 'context', 'events', 'tpls'],
    function ($, _, Backbone, Handlebars, Context, Events, Templates) {
      var TeamMember = Backbone.View.extend({
        tagName: "tr",
        className: "team-member",
        template: Templates["user/team_member"],
        initialize: function (model) {
          _.bindAll(this, "deleteMember", "approveMember");
          this.model = model;
          this.model.on('change', this.render, this);
        },
        render: function () {
          var data = this.model.toJSON();
          this.$el.html(this.template({user: data, is_root: Context.user.get("is_root")}));
          return this;
        },
        events: {
          "click .remove": "deleteMember",
          "click .approve": "approveMember"
        },
        deleteMember: function (e) {
          e.preventDefault();
          if (window.confirm("Are you sure?")) {
            console.log("deleting team member...", this.model.toJSON());
            this.model.destroy();
            this.remove();
          }
        },
        approveMember: function (e) {
          e.preventDefault();

          var user = this.model;
          user.approve(function (res) {
            console.log("[user] approve", res);

            if (res.status && res.status === "success") {
              user.set("approved", true);
            } else if (res.error) {
              alert(res.error);
            }
          });
        }
      });
      return TeamMember;
    });
